##++
##
## Begin Configuration Parameters
##
set managerHost {localhost}
set managerPort {8090}
##
## End of Configuration Parameters -- do not exit below this line
##
##--

set managerSocket {}
set managerConnected 0


###########################################################################
#
# Procedure Name : InitiateConnection
#
# Description : This initiates a connection to the License Manager.
#
#               All the work is done in the file event callbacks.
#
# Arguments : None
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc InitiateConnection {} {
    variable managerSocket
    variable managerHost
    variable managerPort
    variable managerConnected

    set managerSocket [socket -async $managerHost $managerPort]
    fileevent $managerSocket writable AcceptConnection
    fileevent $managerSocket readable \
        [list ConnectionError $managerSocket]
    set managerConnected 0
    return
}



###########################################################################
#
# Procedure Name : AcceptConnection
#
# Description : This accepts a connection and sends the HELLO message.
#
# Arguments : None
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc AcceptConnection {} {
    variable managerSocket
    variable managerConnected

    set message [list HELLO [file tail [info nameofexecutable]]]
    fconfigure $managerSocket -translation binary -blocking 0
    puts -nonewline $managerSocket [format {%10.10d%s} [string bytelength $message] $message]
    flush $managerSocket
    set managerConnected 1
    catch {fileevent $managerSocket writable {}}
    return
}


###########################################################################
#
# Procedure Name : ConnectionError
#
# Description : This is called if we fail to connect or the connection is lost.
#
# Arguments :
#     socket -- The channel name for the socket.
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc ConnectionError {socket} {
    variable managerSocket

    catch {fileevent $socket writable {}}
    catch {fileevent $socket readable {}}
    catch {close $socket}
    if {[string equal $managerSocket $socket]} {
        InitiateConnection
    }
    return
}



###########################################################################
#
# Procedure Name : Disconnect
#
# Description : This informs the liscense manager we exiting.
#               It should be called by an exit handler.
#               This is done automatical for a Tk application.
#
# Arguments : None
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc Disconnect {} {
    variable managerSocket
    variable managerConnected

    if {$managerConnected} {
        set managerConnected 0
        set message DISCONNECT
        catch {
            fileevent $managerSocket writable {}
            fileevent $managerSocket readable {}
            puts -nonewline $managerSocket [format {%10.10d%s} [string bytelength $message] $message]
            flush $managerSocket
            close $managerSocket
        }
    }
    return
}


##
## Start attempting to connect to the license manager
##
InitiateConnection

if {[llength [info command bind]]} {
    bind . <Destroy> +Disconnect
}
